#include <QDebug>
#include "bridge.h"
#include "editor.h"
#include "tools.h"
#include "../common/sources/Map.h"

CBridge     bridge;
extern QQuickItem*  tileSelGrid;
extern QQuickItem*  wallSelGrid;
extern QQuickItem*  objSelGrid;

//---------------------------------------------------------------------------------------------
CBridge::CBridge(QObject *parent) : QObject(parent)
{
    // default values
    mMapSize = CVec2(15, 10);
    mMousePos = CVec2(-1, -1);
    leftPressing = false;
    leftPressed = false;
    rightPressed = false;
    mTabIndex = eTabMap;
    mTool = eQMLToolDraw;
    set_tileSelScrollVis(false);
    set_wallSelScrollVis(false);
    set_objectSelScrollVis(false);
    set_rectSelScrollVis(false);
    set_tileType(1);
    set_wallType(1);
}

void    CBridge::setMapWidth(qint32 valeur)
{
    mMapSize.x = valeur;
}

void    CBridge::setMapHeight(qint32 valeur)
{
    mMapSize.y = valeur;
}

void    CBridge::setMouse(qint32 x, qint32 y)
{
    leftPressing = false;
    mMousePos = CVec2(x, y);

    editor.useTool();
    updateQMLImage();
}

void    CBridge::setMousePress(qint32 button)
{
    leftPressing = false;

    if (button == Qt::LeftButton)
    {
        leftPressing = true;
        leftPressed = true;
    }
    else if (button == Qt::RightButton)
        rightPressed = true;

    editor.useTool();
    updateQMLImage();
}

void    CBridge::setMouseRelease(qint32 button)
{
    leftPressing = false;

    if (button == Qt::LeftButton)
        leftPressed = false;
    else if (button == Qt::RightButton)
        rightPressed = false;

    editor.useTool();
    updateQMLImage();
}

void CBridge::setTabIndex(qint32 index)
{
    editor.unselect();
    mTabIndex = (ETabIndex)index;
}

void    CBridge::resizeMap()
{
    editor.unselect();
    map.mSize = mMapSize;
    map.resize();
    updateQMLImage();
}

void    CBridge::save(QString fileName)
{
    //trick to convert old map formats
#ifdef CONVERT_MAP
//    map.mTilesParams.clear();
//    editor.readTilesDB("databases/tiles.xml");
//    map.mWallsParams.clear();
//    editor.readWallsDB("databases/walls.xml");
#endif

    // [8] to remove "file:///"
    map.save(&fileName.toLocal8Bit().data()[8]);
}

void    CBridge::load(QString fileName)
{
    //trick to convert old map formats
#ifdef CONVERT_MAP
//    map.mTilesParams.clear();
//    editor.readTilesDB("databases/tiles-old.xml");
//    map.mWallsParams.clear();
//    editor.readWallsDB("databases/walls-old.xml");
#endif

    // [8] to remove "file:///"
    map.load(&fileName.toLocal8Bit().data()[8]);
    updateQMLImage();
}

void    CBridge::undo()
{
    editor.mUndoStack->undo();
}

void    CBridge::redo()
{
    editor.mUndoStack->redo();
}

void    CBridge::setTool(qint32 tool)
{
    mTool = (EQMLTools)tool;
    updateQMLImage();
}

void    CBridge::copy()
{
    editor.toolCopy();
}

void    CBridge::cut()
{
    editor.toolCut();
}

void    CBridge::setSelTileType(qint32 type)
{
    drawTileTool*   tool = new drawTileTool();

    if (tool->init(editor.mSelectStart, type) == true)
    {
        editor.mUndoStack->push(tool);
        editor.deleteParamItems(&editor.mTileParamItems);
        editor.addTileParamsItems(type);
    }
    else
    {
        delete tool;
    }
}

void    CBridge::setSelWallType(qint32 type)
{
    drawWallTool*   tool = new drawWallTool();

    if (tool->init(editor.mSelectStart, type) == true)
    {
        editor.mUndoStack->push(tool);
        editor.deleteParamItems(&editor.mWallParamItems);
        editor.addWallParamsItems(type);
    }
    else
    {
        delete tool;
    }
}

void    CBridge::setSelParamCombo(qint32 id, qint32 value)
{
    if (id == -1)
        return;

    if (mTabIndex == eTabTiles)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        CParam*     param = map.getTile(pos)->mParams[id];

        switch (param->mType)
        {
            case eParamOrnate:
                ((CParamOrnate*)param)->mValue = value;
                break;

            case eParamDoor:
                ((CParamDoor*)param)->mValue = value;
                break;

            case eParamDoorOrnate:
                ((CParamDoorOrnate*)param)->mValue = value;
                break;

            default:
                break;
        }
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);
        CWall*      wall = &map.getTile(pos)->mWalls[side];
        CParam*     param = wall->mParams[id];

        switch (param->mType)
        {
            case eParamOrnate:
                ((CParamOrnate*)param)->mValue = value;
                break;

            default:
                break;
        }
    }
}

void    CBridge::setSelParamText(qint32 id, QString value)
{
    if (id == -1)
        return;

    if (mTabIndex == eTabTiles)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        CTile*      tile = map.getTile(pos);
        CParam*     param = tile->mParams[id];

        switch (param->mType)
        {
            case eParamInt:
                ((CParamInt*)param)->mValue = value.toInt();
                break;

            default:
                break;
        }
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);
        CWall*      wall = &map.getTile(pos)->mWalls[side];
        CParam*     param = wall->mParams[id];

        switch (param->mType)
        {
            case eParamInt:
                ((CParamInt*)param)->mValue = value.toInt();
                break;

            default:
                break;
        }
    }
}

void    CBridge::setSelParamBool(qint32 id, bool value)
{
    if (id == -1)
        return;

    if (mTabIndex == eTabTiles)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        CTile*      tile = map.getTile(pos);
        CParam*     param = tile->mParams[id];

        switch (param->mType)
        {
            case eParamBool:
                ((CParamBool*)param)->mValue = value;
                break;

            default:
                break;
        }
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);
        CWall*      wall = &map.getTile(pos)->mWalls[side];
        CParam*     param = wall->mParams[id];

        switch (param->mType)
        {
            case eParamBool:
                ((CParamBool*)param)->mValue = value;
                break;

            default:
                break;
        }
    }
}

//---------------------------------------------------------------------------------------------
MyImageProvider::MyImageProvider() : QQuickImageProvider(QQuickImageProvider::Image)
{
}

QImage MyImageProvider::requestImage(const QString &/*id*/, QSize *size, const QSize &/*requestedSize*/)
{
    QImage image = editor.displayMap(map, bridge.mMousePos, bridge.mTabIndex);

    if (size != NULL)
        *size = QSize(image.width(), image.height());

    return image;
}
